#include "HelloWorldScene.h"
#include "cocostudio/CocoStudio.h"
#include "ui/CocosGUI.h"
#include "MapScene.hpp"
#include "SimpleAudioEngine.h"

using namespace ui;
using namespace cocostudio;

USING_NS_CC;

using namespace cocostudio::timeline;

Scene* HelloWorld::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    //////////////////////////////
    // 1. super init first
    if ( !Layer::init() )
    {
        return false;
    }
    
    HelloWorld::playBGM();
    
    m_rootNode = CSLoader::createNode("MainScene.csb");

    addChild(m_rootNode);
    
    Button *helpbtn = static_cast<Button *>(m_rootNode->getChildByName("help"));
    helpbtn->addClickEventListener(CC_CALLBACK_1(HelloWorld::onHelpbtn, this));

    Button *mapbtn = static_cast<Button *>(m_rootNode->getChildByName("start"));
    mapbtn->addClickEventListener(CC_CALLBACK_1(HelloWorld::onMapbtn, this));

    SpriteFrameCache::getInstance()->addSpriteFramesWithFile("./edison.plist");
    auto d_down = HelloWorld::createAnimation(AMINATION_EDISON_DOWN, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(d_down, AMINATION_EDISON_DOWN);
    auto d_left = HelloWorld::createAnimation(AMINATION_EDISON_LEFT, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(d_left, AMINATION_EDISON_LEFT);
    auto d_right = HelloWorld::createAnimation(AMINATION_EDISON_RIGHT, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(d_right, AMINATION_EDISON_RIGHT);
    auto d_up = HelloWorld::createAnimation(AMINATION_EDISON_UP, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(d_up, AMINATION_EDISON_UP);
    auto d_die = HelloWorld::createAnimation(AMINATION_EDISON_DIE, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(d_die, AMINATION_EDISON_DIE);

    SpriteFrameCache::getInstance()->addSpriteFramesWithFile("./tesla.plist");
    auto t_down = HelloWorld::createAnimation(AMINATION_TESLA_DOWN, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(t_down, AMINATION_TESLA_DOWN);
    auto t_left = HelloWorld::createAnimation(AMINATION_TESLA_LEFT, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(t_left, AMINATION_TESLA_LEFT);
    auto t_right = HelloWorld::createAnimation(AMINATION_TESLA_RIGHT, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(t_right, AMINATION_TESLA_RIGHT);
    auto t_up = HelloWorld::createAnimation(AMINATION_TESLA_UP, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(t_up, AMINATION_TESLA_UP);
    auto t_die = HelloWorld::createAnimation(AMINATION_TESLA_DIE, 4, 0.2f);
    AnimationCache::getInstance()->addAnimation(t_die, AMINATION_TESLA_DIE);

    return true;
}

void HelloWorld::onHelpbtn(Ref *pSender) {
    HelloWorld::playSE("click.mp3");
    auto helpbk = static_cast<ImageView *>(m_rootNode->getChildByName("helpbk"));
    if (helpbk->isVisible()) {
        helpbk->setVisible(false);
    } else {
        helpbk->setVisible(true);
    }
}

void HelloWorld::onMapbtn(Ref *pSender) {
    HelloWorld::playSE("click.mp3");
    Director::getInstance()->setDepthTest(true);
    Scene *mapscene = Scene::create();
    CMapScene* map = CMapScene::create();
    mapscene->addChild(map);
    auto ss = TransitionProgressInOut::create(1, mapscene);
    Director::getInstance()->replaceScene(ss);
}

Animation* HelloWorld::createAnimation(string prefixName, int framesNum, float delay) {
    Vector<SpriteFrame*> animFrames;
    
    for (int i=0;i<framesNum;i++) {
        string str = StringUtils::format("%s%i.png", prefixName.c_str(), i);
        auto frame = SpriteFrameCache::getInstance()->getSpriteFrameByName(str);
        animFrames.pushBack(frame);
    }
    return Animation::createWithSpriteFrames(animFrames, delay);
}

void HelloWorld::playBGM() {
    auto en = CocosDenshion::SimpleAudioEngine::getInstance();
    if (en->isBackgroundMusicPlaying()) {
        en->stopBackgroundMusic();
    }
    en->playBackgroundMusic("music.mp3", true);
}

void HelloWorld::playSE(string path) {
    auto en = CocosDenshion::SimpleAudioEngine::getInstance();
    en->playEffect(path.c_str());
}


