
#include "Map.hpp"
#include "MapScene.hpp"

CMap::CMap(void *parent, bool small) {
    m_parent = parent;
    m_small = small;
    
    m_cont = NULL;
    m_map = NULL;
    
    m_tiles = NULL;

    m_roleE = NULL;
    m_roleT = NULL;
    
    m_cf = new __CCCallFuncND();
}

CMap::~CMap() {
    if (m_cf != NULL) {
        delete m_cf;
    }
    delete m_roleE;
    delete m_roleT;
    
    for(int i=0;i<m_wid;i++) {
        for (int j=0;j<m_hei;j++) {
            delete m_tiles[i][j];
        }
        delete []m_tiles[i];
    }
    delete []m_tiles;

}

bool CMap::createMap(ScrollView *scroll) {
    m_cont = scroll;
    m_cont->setSwallowTouches(false);
    m_cont->setInertiaScrollEnabled(false);
    m_cont->setScrollBarEnabled(false);
    m_map = (TMXTiledMap *)m_cont->getChildByName("map");

    TMXLayer *tlayer = m_map->getLayer("bk");
    auto elc = m_map->getLayer("elc");
    if (tlayer == NULL) {
        return false;
    }
    m_wid = tlayer->getLayerSize().width;
    m_hei = tlayer->getLayerSize().height;
    m_tiles = new CTile**[m_wid];
    for(int i = 0;i<m_wid;i++) {
        m_tiles[i] = new CTile*[m_hei];
        for(int j=0;j<m_hei;j++) {
            m_tiles[i][j] = new CTile();
        }
    }
    for (int i=0;i<m_wid;i++) {
        for (int j=0;j<m_hei;j++) {
            unsigned int gid = tlayer->getTileGIDAt(Point(i,j));
            if (gid > 0) {
                auto tiledic = m_map->getPropertiesForGID(gid);
                const int type = tiledic.asValueMap()["type"].asInt();
                const int block = tiledic.asValueMap()["block"].asInt();
                m_tiles[i][j]->setTile((TTerrain)block, (TType)type);
            }
            gid = elc->getTileGIDAt(Point(i,j));
            if (gid > 0) {
                auto tiledic = m_map->getPropertiesForGID(gid);
                const int type = tiledic.asValueMap()["type"].asInt();
                const int side = tiledic.asValueMap()["side"].asInt();
                if (type == BTYPE_TOWER) {
                    CTower *tower = ((CMapScene *)m_parent)->getTowerByCoord(Vec2(i,j));
                    if (tower == NULL) {
                        tower = new CTower();
                        tower->init(Vec2(i,j), side>0?false:true);
                        m_tiles[i][j]->setTower(tower);
                        ((CMapScene *)m_parent)->addTower(tower);
                    } else if (m_small) {
                        setElc(BTYPE_FLAG_TOWER, Vec2(i,j), side>0?false:true);
                    }
                }
            }
        }
    }
    TMXObjectGroup *og = m_map->getObjectGroup("obj");
    ValueVector objs = og->getObjects();
    for (auto iter = objs.begin();iter!=objs.end();iter++) {
        ValueMap vm = iter->asValueMap();
        int type = vm["type"].asInt();
        int x = vm["x"].asInt();
        int y = vm["y"].asInt();
        auto node = CSLoader::createNode("RoleNode.csb");
        Vec2 tilecoord = convertToTileCoord(Vec2(x,y));
        Vec2 newpos = convertToPosition(tilecoord);
        if (type == 0) {
            if (m_roleE != NULL) {
                delete m_roleE;
                m_roleE = NULL;
            }
            if (m_small) {
                setElc(BTYPE_FLAG_ROLE, tilecoord, true);
            } else {
                m_roleE = new CRole();
                m_roleE->createRole(node, m_map, true, newpos, tilecoord);
            }
        } else {
            if (m_roleT != NULL) {
                delete m_roleT;
                m_roleT = NULL;
            }
            if (m_small) {
                setElc(BTYPE_FLAG_ROLE, tilecoord, false);
            } else {
                m_roleT = new CRole();
                m_roleT->createRole(node, m_map, false, newpos, tilecoord);
            }
        }
    }
    return true;
}

Vec2 CMap::getMovablePos(Vec2 pos, MEAction dir) {
    Vec2 ret = pos;
    switch (dir) {
        case ACTION_DOWN:
            ret.y++;
            break;
        case ACTION_UP:
            ret.y--;
            break;
        case ACTION_LEFT:
            ret.x--;
            break;
        case ACTION_RIGHT:
            ret.x++;
            break;
        default:
            break;
    }
    if ((ret.x < 0)||(ret.x >= m_wid)||(ret.y < 0)||(ret.y >= m_hei)) {
        return pos;
    }
    CTile *tile = getTileByCoord(ret);
    if (tile == NULL) {
        return pos;
    }
    if (tile->isBlock()) {
        return pos;
    }
    return ret;
}

bool CMap::moveRole(bool edison, MEAction action, bool conmove) {
    if (((CMapScene *)m_parent)->isResult()) {
        return false;
    }
    Vec2 oldcoord;
    int side = -1;
    if (edison) {
        oldcoord = m_roleE->getCoord();
        Vec2 newcoord = getMovablePos(oldcoord, action);
        if (m_roleT->getCoord() == newcoord) {
            return false;
        }
        Vec2 newpos = convertToPosition(newcoord);
        m_cf->initWithTarget(this, callfuncND_selector(CMap::callback), (void*)m_roleE);
        if ((m_roleE->moveOnce(newcoord, newpos, action, m_cf))&&conmove) {
            //log("Move:(%d/%d)", (int)newpos.x, (int)newpos.y);
            setConPosition(action);
            m_othermap->moveRole(true, action, false);
            //纠正位置
            CRole *orole = m_othermap->getRole(true);
            if (m_roleE->getCoord() != orole->getCoord()) {
                Vec2 op = convertToPosition(m_roleE->getCoord());
                orole->setCP(op, m_roleE->getCoord());
            }
            side = isInLine(oldcoord);
            if ((((CMapScene *)m_parent)->getTowerByCoord(oldcoord) == NULL)&&(side < 0)) {
                m_smallmap->delElc(oldcoord);
            }
            side = isInLine(m_roleE->getCoord());
            if ((((CMapScene *)m_parent)->getTowerByCoord(m_roleE->getCoord()) == NULL)&&(side < 0)) {
                m_smallmap->setElc(BTYPE_FLAG_ROLE, m_roleE->getCoord(), true);
            } else if (side > 0) {  //爱迪生在特斯拉的电流中
                m_roleE->die();
                m_othermap->getRole(true)->die();
                ((CMapScene *)m_parent)->showWinLayer(false);
            }
            return true;
        }
    } else {
        oldcoord = m_roleT->getCoord();
        Vec2 newcoord = getMovablePos(oldcoord, action);
        if (m_roleE->getCoord() == newcoord) {
            return false;
        }
        Vec2 newpos = convertToPosition(newcoord);
        m_cf->initWithTarget(this, callfuncND_selector(CMap::callback), (void*)m_roleT);
        if ((m_roleT->moveOnce(newcoord, newpos, action, m_cf))&&conmove) {
            //log("Move:(%d/%d)", (int)newpos.x, (int)newpos.y);
            setConPosition(action);
            m_othermap->moveRole(false, action, false);
            //纠正位置
            CRole *orole = m_othermap->getRole(false);
            if (m_roleT->getCoord() != orole->getCoord()) {
                Vec2 op = convertToPosition(m_roleT->getCoord());
                orole->setCP(op, m_roleT->getCoord());
            }
            side = isInLine(oldcoord);
            if ((((CMapScene *)m_parent)->getTowerByCoord(oldcoord) == NULL)&&(side < 0)) {
                m_smallmap->delElc(oldcoord);
            }
            side = isInLine(m_roleT->getCoord());
            if ((((CMapScene *)m_parent)->getTowerByCoord(m_roleT->getCoord()) == NULL)&&(side < 0)) {
                m_smallmap->setElc(BTYPE_FLAG_ROLE, m_roleT->getCoord(), false);
            } else if (side == 0) {  //特斯拉在爱迪生的电流中
                m_roleT->die();
                m_othermap->getRole(false)->die();
                ((CMapScene *)m_parent)->showWinLayer(true);
            }
            return true;
        }
    }
    return false;
}

bool CMap::setElc(BType type, Vec2 coord, bool edison) {
    if ((coord.x < 0)||(coord.x >= m_wid)||(coord.y < 0)||(coord.y >= m_hei)) {
        return false;
    }
    if (m_tiles[(int)coord.x][(int)coord.y] == NULL) {
        return false;
    }
    if (m_tiles[(int)coord.x][(int)coord.y]->isBlock()) {
        return false;
    }
    
    TMXLayer *tlayer = m_map->getLayer("elc");
    if (tlayer == NULL) {
        return false;
    }
    TMXTilesetInfo *tinfo = tlayer->getTileSet();
    int id = type * 2 + (edison?0:1);
    int index = tinfo->_firstGid+id;
    tlayer->setTileGID(index, coord);
    //log("SetElc %d:(%d,%d)", type, (int)coord.x, (int)coord.y);
    return true;
}

bool CMap::delElc(Vec2 coord) {
    TMXLayer *tlayer = m_map->getLayer("elc");
    if (tlayer == NULL) {
        return false;
    }
    tlayer->removeTileAt(coord);
    return true;
}

int CMap::isInLine(Vec2 coord) {
    TMXLayer *tlayer = m_map->getLayer("elc");
    if (tlayer == NULL) {
        return -1;
    }
    unsigned int gid = tlayer->getTileGIDAt(coord);
    if (gid > 0) {
        auto tiledic = m_map->getPropertiesForGID(gid);
        const int type = tiledic.asValueMap()["type"].asInt();
        const int side = tiledic.asValueMap()["side"].asInt();
        if (m_small) {
            if ((type == BTYPE_FLAG_EH)||(type == BTYPE_FLAG_EV)) { //电流
                return side;
            }
        } else {
            if (type == BTYPE_ELC) {    //电流
                return side;
            }
        }
    }
    return -1;
}

CTile *CMap::getTileByCoord(Vec2 coord) {
    if ((coord.x < 0)||(coord.y < 0)||(coord.x >= m_wid)||(coord.y >= m_hei)) {
        return NULL;
    }
    return m_tiles[(int)coord.x][(int)coord.y];
}

void CMap::callback(Node* sender, void* data) {
    if ((m_parent == NULL)||(data == NULL)) {
        return ;
    }
    ((CRole *)data)->setState(STATE_IDLE);
    CTile *tile = getTileByCoord(((CRole *)data)->getCoord());
    if (tile != NULL) {
        Vec2 coord = ((CRole *)data)->getCoord();
        bool edison = ((CRole *)data)->isEdison();
        bool ct = false;
        if (((CMapScene *)m_parent)->addTower(coord, edison)) {
            if (((CMapScene *)m_parent)->lineInOne(edison)) { //判断是否连成一条线
                ct = true;
                ((CMapScene *)m_parent)->showWinLayer(edison);
            }
        }
        m_smallmap->updateLines(coord); //更新小地图
        m_othermap->updateLines(coord); //更新另一个地图
        if (!ct&&updateLines(coord)) { //获胜
            if (edison) {
                m_roleT->die();
                m_othermap->getRole(false)->die();
            } else {
                m_roleE->die();
                m_othermap->getRole(true)->die();
            }
            ((CMapScene *)m_parent)->showWinLayer(edison);
        }
    }
}

Vec2 CMap::convertToTileCoord(Vec2 pos) {
    Size ts = m_map->getTileSize();
    int x = pos.x / ts.width;
    int y = pos.y / ts.height;
    return Vec2(x,y);
}

Vec2 CMap::convertToPosition(Vec2 coord) {
    Size ts = m_map->getTileSize();
    Size ms = m_map->getMapSize();
    int x = (int)coord.x * (int)ts.width + (int)ts.width/2;
    int y = ((int)ms.height - (int)coord.y) * (int)ts.height - (int)ts.height/2;
    return Vec2(x,y);
}

bool CMap::setConPosition(MEAction action) {
    Vec2 pic = m_map->getTileSize();
    Vec2 pos = m_cont->getInnerContainer()->getPosition();
    switch (action) {
        case ACTION_DOWN:
            pos.y+=pic.y;
            break;
        case ACTION_UP:
            pos.y-=pic.y;
            break;
        case ACTION_LEFT:
            pos.x+=pic.x;
            break;
        case ACTION_RIGHT:
            pos.x-=pic.x;
            break;
    }
    MoveTo *mt = MoveTo::create(0.4f, pos);
    m_cont->getInnerContainer()->runAction(Sequence::create(mt, NULL, NULL));    //移动实体对象

    return true;
}

bool CMap::updateLines(Vec2 coord) {
    bool ret = false;
    if (updateDirectLine(coord, ACTION_DOWN)) {
        ret = true;
    }
    if (updateDirectLine(coord, ACTION_UP)) {
        ret = true;
    }
    if (updateDirectLine(coord, ACTION_LEFT)) {
        ret = true;
    }
    if (updateDirectLine(coord, ACTION_RIGHT)) {
        ret = true;
    }
    return ret;
}

bool CMap::updateDirectLine(Vec2 coord, MEAction dir) {
    CTower *tower = ((CMapScene *)m_parent)->getTowerByCoord(coord);
    if (tower == NULL) {
        return false;
    }
    bool te = tower->isEdisons();
    CTower *neartower = ((CMapScene *)m_parent)->getTowerByDirect(coord, dir);
    if (neartower == NULL) {
        return false;
    }
    bool win = false;
    bool isv = false;
    bool ne = neartower->isEdisons();
    for (int i=0;i<TILE_TOWER_DISTANCE-1;i++) {
        switch (dir) {
            case ACTION_DOWN:
                coord.y++;
                isv = true;
                break;
            case ACTION_UP:
                coord.y--;
                isv = true;
                break;
            case ACTION_LEFT:
                coord.x--;
                break;
            case ACTION_RIGHT:
                coord.x++;
                break;
        }
        if (te == ne) {
            if (te) {   //当前是爱迪生的电塔
                if (!m_small&&(m_roleT->getCoord() == coord)) {
                    win = true;
                }
            } else {    //当前是特斯拉的电塔
                if (!m_small&&(m_roleE->getCoord() == coord)) {
                    win = true;
                }
            }
            if (m_small) {
                setElc(isv?BTYPE_FLAG_EV:BTYPE_FLAG_EH, coord, te);
            } else {
                setElc(BTYPE_ELC, coord, te);
            }
        } else {
            delElc(coord);
        }
    }
    return win;
}

CRole *CMap::getRole(bool edison) {
    return edison?m_roleE:m_roleT;
}

