
#include "MapScene.hpp"
#include "HelloWorldScene.h"
#include "ui/CocosGUI.h"
#include "cocostudio/CocoStudio.h"

Scene* CMapScene::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = CMapScene::create();
    
    // add layer as a child to scene
    scene->addChild(layer);
    
    // return the scene
    return scene;
}

CMapScene::~CMapScene() {
    unschedule(schedule_selector(CMapScene::onTimer));
    
    delete m_mapE;
    delete m_mapT;
    delete m_mapS;
    
    for (auto iter = m_towers.begin();iter!=m_towers.end();iter++) {
        delete *iter;
    }
    m_towers.clear();
    delete m_winlayer;
}

bool CMapScene::init()
{
    //////////////////////////////
    // 1. super init first
    if (!Layer::init())
    {
        return false;
    }
    
    m_rootNode = CSLoader::createNode("MapScene.csb");
    
    addChild(m_rootNode);
    
    m_mapE = new CMap(this, false);
    m_mapT = new CMap(this, false);
    m_mapS = new CMap(this, true);
    
    auto sw0 = (ScrollView *)m_rootNode->getChildByName("c0");
    m_mapE->createMap(sw0);

    auto sw1 = (ScrollView *)m_rootNode->getChildByName("c1");
    m_mapT->createMap(sw1);
    
    auto sw2 = (ScrollView *)m_rootNode->getChildByName("c2");
    m_mapS->createMap(sw2);

    m_mapE->setOtherMap(m_mapT, m_mapS);
    m_mapT->setOtherMap(m_mapE, m_mapS);

    // 创建键盘监听器
    auto listener = EventListenerKeyboard::create();
    listener->onKeyPressed = CC_CALLBACK_2(CMapScene::onKeyPressed, this);
    //listener->onKeyReleased = CC_CALLBACK_2(KeyboardTest::onKeyReleased, this);
    _eventDispatcher->addEventListenerWithSceneGraphPriority(listener, this);

    updateTowerCount(); //更新电塔数量
    
    auto td = (ScrollView *)m_rootNode->getChildByName("winbk");
    m_winlayer = new CWinLayer(td, this);
    m_winlayer->setNodeVisible(false);
    
    m_time = GAME_TIME;
    auto sched = Director::getInstance()->getScheduler();
    sched->schedule(schedule_selector(CMapScene::onTimer), this, 1.0f, false);
    return true;
}

void CMapScene::onKeyPressed(EventKeyboard::KeyCode keyCode, Event* event) {
    switch (keyCode) {
        case EventKeyboard::KeyCode::KEY_W:   //p0上
            m_mapE->moveRole(true, ACTION_UP, true);
            break;
        case EventKeyboard::KeyCode::KEY_S:   //p0下
            m_mapE->moveRole(true, ACTION_DOWN, true);
            break;
        case EventKeyboard::KeyCode::KEY_A:   //p0左
            m_mapE->moveRole(true, ACTION_LEFT, true);
            break;
        case EventKeyboard::KeyCode::KEY_D:   //p0右
            m_mapE->moveRole(true, ACTION_RIGHT, true);
            break;
        case EventKeyboard::KeyCode::KEY_UP_ARROW:   //p1上
            m_mapT->moveRole(false, ACTION_UP, true);
            break;
        case EventKeyboard::KeyCode::KEY_DOWN_ARROW:   //p1下
            m_mapT->moveRole(false, ACTION_DOWN, true);
            break;
        case EventKeyboard::KeyCode::KEY_LEFT_ARROW:   //p1左
            m_mapT->moveRole(false, ACTION_LEFT, true);
            break;
        case EventKeyboard::KeyCode::KEY_RIGHT_ARROW:   //p1右
            m_mapT->moveRole(false, ACTION_RIGHT, true);
            break;
        default:
            break;
    }
}

bool CMapScene::showWinLayer(bool edison) {
    if (m_winlayer == NULL) {
        return false;
    }
    unschedule(schedule_selector(CMapScene::onTimer));
    return m_winlayer->showLayer(edison);
}

bool CMapScene::isResult() {
    if (m_winlayer == NULL) {
        return false;
    }
    return m_winlayer->isNodeVisible();
}

void CMapScene::onReturn(Ref *pSender) {
    HelloWorld::playSE("click.mp3");
    Director::getInstance()->setDepthTest(true);
    Scene *mainscene = Scene::create();
    HelloWorld* main = HelloWorld::create();
    mainscene->addChild(main);
    auto ss = TransitionProgressRadialCW::create(1, mainscene);
    Director::getInstance()->replaceScene(ss);
}

CTower *CMapScene::addTower(Vec2 coord, bool edison) {
    CTile *tile = m_mapE->getTileByCoord(coord);
    if ((tile == NULL)||(!tile->canBuild(edison))) {
        return NULL;
    }
    //删除之前的电塔
    CTower *tower = NULL;
    for (auto iter = m_towers.begin();iter!=m_towers.end();iter++) {
        if ((*iter)->getCoord() == coord) {
            tower = *iter;
            m_towers.erase(iter);
            break;
        }
    }
    if (tower != NULL) {
        delete tower;
    }
    //创建新的电塔
    tower = new CTower();
    tower->init(coord, edison);
    if (!tile->setTower(tower)) {
        delete tower;
    }
    m_mapE->setElc(BTYPE_TOWER, coord, edison);
    m_mapT->setElc(BTYPE_TOWER, coord, edison);
    m_mapS->setElc(BTYPE_FLAG_TOWER, coord, edison);
    m_towers.push_back(tower);
    updateTowerCount(); //更新电塔数量
    HelloWorld::playSE("tower.mp3");
    return tower;
}

bool CMapScene::addTower(CTower *tower) {
    if (tower == NULL) {
        return false;
    }
    m_towers.push_back(tower);
    return true;
}

int CMapScene::updateTowerCount() {
    auto p0 = (Text *)m_rootNode->getChildByName("p0");
    auto p1 = (Text *)m_rootNode->getChildByName("p1");
    int ec = 0;
    int tc = 0;
    for (auto iter = m_towers.begin();iter!=m_towers.end();iter++) {
        if ((*iter)->isEdisons()) {
            ec++;
        } else {
            tc++;
        }
    }
    p0->setString(StringUtils::format("%d", ec));
    p1->setString(StringUtils::format("%d", tc));
    if (ec > tc) {
        return 1;
    } else if (ec < tc) {
        return -1;
    }
    return 0;
}

CTower *CMapScene::getTowerByCoord(Vec2 coord) {
    for (auto iter = m_towers.begin();iter!=m_towers.end();iter++) {
        if ((*iter)->getCoord() == coord) {
            return *iter;
        }
    }
    return NULL;
}

CTower *CMapScene::getTowerByDirect(Vec2 coord, MEAction dir) {
    switch (dir) {
        case ACTION_RIGHT:
            coord.x+=TILE_TOWER_DISTANCE;
            break;
        case ACTION_LEFT:
            coord.x-=TILE_TOWER_DISTANCE;
            break;
        case ACTION_UP:
            coord.y-=TILE_TOWER_DISTANCE;
            break;
        case ACTION_DOWN:
            coord.y+=TILE_TOWER_DISTANCE;
            break;
    }
    if ((coord.x < 0)||(coord.y < 0)) {
        return NULL;
    }
    return getTowerByCoord(coord);
}

void CMapScene::onTimer(float fDelta) {
    m_time--;
    auto text = (Text *)m_rootNode->getChildByName("time");
    text->setString(StringUtils::format("%02d:%02d", m_time/60, m_time%60));
    if (m_time <= 0) {
        int ret = updateTowerCount();
        m_winlayer->showLayer(ret>0?true:false);
        unschedule(schedule_selector(CMapScene::onTimer));
        return ;
    }
}

bool CMapScene::lineInOne(bool edison) {
    for (int i=7;i<32;i+=6) {
        for (int t=2;t<7;t++) {
            const int rows = 10, cols = 10;
            int nowPos, prePos;
            bool finds = false;
            stack<int> path;
            int record[rows*cols];
            int mapIndex[rows*cols] = {
                0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,
                2,0,0,0,0,0,0,0,0,2,
                2,0,0,0,0,0,0,0,0,2,
                2,0,0,0,0,0,0,0,0,2,
                2,0,0,0,0,0,0,0,0,2,
                2,0,0,0,0,0,0,0,0,2,
                0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0,
                0,0,0,0,0,0,0,0,0,0
            };
            Vec2 hash = convertToHashCoord(Vec2(0,i));
            for (int m=2;m<7;m++) { //把左边其余的入口封死
                int x0 = rows * (int)hash.y + (int)hash.x;
                int x1 = rows * m;
                bool lb = !edison;
                CTower *tw = getTowerByCoord(Vec2(0,i));
                if ((tw != NULL)&&(tw->isEdisons() == edison)) {
                    lb = edison;
                }
                if ((x0 != x1)||(lb != edison)) {
                    mapIndex[x1] = 0;
                }
            }
            for (int m=2;m<7;m++) { //根据i依次封死右边的4个出口
                int x0 = rows * m + 9;
                if (t != m) {
                    mapIndex[x0] = 0;
                } else {
                    CTower *tw = getTowerByCoord(Vec2(54,(t-1)*6+1));
                    if ((tw != NULL)&&(tw->isEdisons() == edison)) {
                        mapIndex[x0] = 3;
                    } else {
                        mapIndex[x0] = 0;
                    }
                }
            }
            for (auto iter = m_towers.begin();iter!=m_towers.end();iter++) {    //根据对象来修改index为1的节点
                if (edison == (*iter)->isEdisons()) {
                    Vec2 cd = (*iter)->getCoord();
                    if ((cd.x > 0)&&(cd.x < 54)) {
                        hash = convertToHashCoord(cd);
                        int x0 = rows * (int)hash.y + (int)hash.x;
                        mapIndex[x0] = 1;
                    }
                }
            }
            //算法计算连通
            for (int k=0;k<rows*cols;k++) {
                record[k] = mapIndex[k];
                if(mapIndex[k] > 0) {   //可通过
                    if(mapIndex[k] == 2) { //找到迷宫入口
                        nowPos = k;
                        path.push(k);
                        record[k] = 0;
                    }
                }
            }
            prePos = cols*rows+1;
            while (!finds) {
                int rowNum;
                int up, down, left, right;
                rowNum = nowPos/cols;
                up = nowPos - cols;
                down = nowPos + cols;
                left = nowPos - 1;
                right = nowPos + 1;
                if ((up >=0)&&record[up]) { //往上走
                    path.push(up);
                    record[up] = 0;
                    prePos = nowPos;
                    nowPos = up;
                    if (mapIndex[nowPos] == 3) { //找到出口
                        finds = true;
                    }
                } else if (down<=cols*rows-1&&record[down]) { //往下走
                    path.push(down);
                    record[down] = 0;
                    prePos = nowPos;
                    nowPos = down;
                    if (mapIndex[nowPos]==3) {
                        finds = true;
                    }
                } else if (left>=rowNum*cols&&record[left]) { //往左走
                    path.push(left);
                    record[left] = 0;
                    prePos = nowPos;
                    nowPos = left;
                    if (mapIndex[nowPos]==3) {
                        finds = true;
                    }
                } else if (right<=(rowNum+1)*cols-1&&record[right]) { //往右走
                    path.push(right);
                    record[right] = 0;
                    prePos = nowPos;
                    nowPos = right;
                    if (mapIndex[nowPos] == 3) {
                        finds = true;
                    }
                } else { //无可移动方格
                    if (path.size()<=1) {
                        break;
                    } else {
                        path.pop();
                        prePos = nowPos;
                        nowPos = path.top();
                    }
                }
            }
            if (finds) {
                return true;
            }
        }
    }
    return false;
}

Vec2 CMapScene::convertToHashCoord(Vec2 coord) {
    Vec2 ret;
    ret.x = (int)coord.x / 6;
    ret.y = (int)coord.y /6 + 1;
    return ret;
}



