
#include "Role.hpp"
#include "HelloWorldScene.h"
#include "ui/CocosGUI.h"
#include "cocostudio/CocoStudio.h"

CRole::CRole() {
    m_node = NULL;
    m_man = true;
    m_state = STATE_IDLE;
}

bool CRole::createRole(Node *node, TMXTiledMap *map, bool edison, Vec2 pos, Vec2 coord) {
    if (map == NULL) {
        return false;
    }
    m_node = CSLoader::createNode("RoleNode.csb");
    if (m_node == NULL) {
        return false;
    }
    auto sp = static_cast<Sprite *>(m_node->getChildByName("role"));
    string str = StringUtils::format("%s%d.png", edison?AMINATION_EDISON_DOWN:AMINATION_TESLA_DOWN, 0);
    sp->setSpriteFrame(str);
    
    m_map = map;
    m_man = edison;
    m_coord = coord;
    
    m_node->setPosition(pos);
    map->addChild(m_node, Z_ORDER_ROLE);
    return true;
}

bool CRole::moveOnce(Vec2 coord, Vec2 pos, MEAction action, CallFunc *callback) {
    //CGameSceneManager::g_gsm->getGameSoundManager()->playSE_Other(terrain->isWater()?SE_NAVY:SE_MOVE);
    if (!setAction(action)||(coord == m_coord)) { //播放对应动作
        m_state = STATE_IDLE;
        return false;
    }
    MoveTo *mt = MoveTo::create(0.4f, pos);
    m_node->runAction(Sequence::create(mt, callback, NULL));    //移动实体对象
    m_coord = coord;
    
    return true;
}

bool CRole::setAction(MEAction action) {
    if ((m_node == NULL)||(m_state != STATE_IDLE)) {
        return false;
    }
    string str;
    auto sp = static_cast<Sprite *>(m_node->getChildByName("role"));
    if (m_man) {
        switch (action) {
            case ACTION_UP:
                str = AMINATION_EDISON_UP;
                break;
            case ACTION_DOWN:
                str = AMINATION_EDISON_DOWN;
                break;
            case ACTION_LEFT:
                str = AMINATION_EDISON_LEFT;
                break;
            case ACTION_RIGHT:
                str = AMINATION_EDISON_RIGHT;
                break;
        }
    } else {
        switch (action) {
            case ACTION_UP:
                str = AMINATION_TESLA_UP;
                break;
            case ACTION_DOWN:
                str = AMINATION_TESLA_DOWN;
                break;
            case ACTION_LEFT:
                str = AMINATION_TESLA_LEFT;
                break;
            case ACTION_RIGHT:
                str = AMINATION_TESLA_RIGHT;
                break;
        }
    }
    m_state = STATE_MOVE;
    sp->runAction(Animate::create(AnimationCache::getInstance()->getAnimation(str)));
    return true;
}

bool CRole::setCP(Vec2 pos, Vec2 coord) {
    pos.x+=36;
    pos.y+=36;
    m_node->setPosition(pos);
    m_coord = coord;
    return true;
}

bool CRole::die() {
    if ((m_node == NULL)||(m_state == STATE_DIE)) {
        return false;
    }
    auto sp = static_cast<Sprite *>(m_node->getChildByName("role"));
    m_state = STATE_DIE;
    sp->stopAllActions();
    sp->runAction(Animate::create(AnimationCache::getInstance()->getAnimation(m_man?AMINATION_EDISON_DIE:AMINATION_TESLA_DIE)));
    return true;
}
